@@ -1,10 +1,9 @@
-from ast import Raise
 import os
 import matplotlib.ticker as mticker
 import sympy as sy
 import matplotlib as mpl
 from math import floor, log10, ceil
-from locale import setlocale, LC_NUMERIC
+from scipy.integrate import quad
 import numpy as np
 
 
@@ -115,7 +114,7 @@ def Find_nearest(array, value):
     idx = (np.abs(array - value)).argmin()
 
     # Return the nearest value
-    return array[idx]
+    return array[idx], idx
 
 
 
@@ -341,3 +340,206 @@ class Plotter():
         # Set label using scientific notation
         elif tenpower != 0:
             ax.set_zlabel(f"${physical_quantity}$" + "$\cdot 10^{" + str(tenpower) + "}$" +  f"[{unit}]", rotation=rot)
+
+
+
+def Fourier_series_cos_term(func, k, omega_0):
+    """
+    Decorator to convert any function to a intergrant for a fourier series (cos)
+
+    Input:
+        func (python function object): A function that needs te be converted
+        k (int): The index of the fourier sereies
+        omega_0 (float): The ground radial velocity
+
+    Output:
+        A function multiplied by the cosine term in a fourier series 
+    """
+
+    # Create a wrapper and take all arguments from the function
+    def wrapper(*args, **kwargs):
+        # Convert  the function by passing all arguments in to the function 
+        # Then multipli the function by cosine and give it as input the 
+        # The index multiplied by the ground radial velocity and t (args[0] must be the variable)
+        result = func(*args, **kwargs)*np.cos(k*omega_0*args[0])
+
+        # Return the result of the function
+        return result
+
+    # Return the wrapper as the new function
+    return wrapper
+
+
+def Fourier_series_sin_term(func, k, omega_0):
+    """
+    Decorator to convert any function to a intergrant for a fourier series (sin)
+
+    Input:
+        func (python function object): A function that needs te be converted
+        k (int): The index of the fourier sereies
+        omega_0 (float): The ground radial velocity
+
+    Output:
+        A function multiplied by the sine term in a fourier series 
+    """
+
+    # Create a wrapper and take all arguments from the function
+    def wrapper(*args, **kwargs):
+        # Convert  the function by passing all arguments in to the function 
+        # Then multipli the function by cosine and give it as input the 
+        # The index multiplied by the ground radial velocity and t (args[0] must be the variable)
+        result = func(*args, **kwargs)*np.sin(k*omega_0*args[0])
+
+        # Return the result of the function
+        return result
+
+    # Return the wrapper as the new function
+    return wrapper
+
+def Fourier_series(func, T, n):
+    """
+    A function to calculate the coefficients of the fourier series for any function numerically
+    Input:
+        func (python function object): The function of which the fourier series has te be
+            calculated
+        T (float): The period of the 
+        n (int): the amount of terms of the fourier series
+    """
+
+    # Calculate the ground radial velocity
+    omega_0 = 2*np.pi/T
+
+    # Create arrays for the coefficients of the fourier series
+    array_a_n = []
+    array_b_n = []
+
+    # Calculate a_0 of the fourier series
+    a_0 = 1/T*quad(func, 0, T)[0]
+
+    # Loop over the terms
+    for k in range(n):
+
+        # Convert the function given to the intergrand for the value k
+        func_cos = Fourier_series_cos_term(func, omega_0, k)
+        func_sin = Fourier_series_sin_term(func, omega_0, k)
+
+        # Calculate the intergral using scipy 
+        a_n = 2/T*quad(func_cos, 0, np.pi)[0]
+        b_n = 2/T*quad(func_sin, 0, np.pi)[0]
+
+        # Append the coefficients to there respected arrays
+        array_a_n.append(a_n)
+        array_b_n.append(b_n)
+
+    # Return a_0 and the coefficient arrays
+    return a_0, array_a_n, array_b_n
+
+def Standaard_error_per_index(*arrays):
+    """
+    A function to calculate the standaard error per index
+
+    Input (list or numpy array): Lists for wich the standaard error has 
+    to be calculeted per index.
+
+    Output (umpy array): A list of standaard errors per index
+    """
+
+    # Create a matrix with the given arrays
+    M = np.array([*arrays])
+
+    # Determine the shape of the matrix 
+    rows = M.shape[0]
+    columns = M.shape[1]
+    
+    # Create a array to store the std per column
+    E = np.array([])
+
+    # Loop over all the columns in the matrix
+    for i in range(columns):
+
+        # Calculate the std for all columns
+        s = np.std(M[:,i], ddof=1)
+
+        # Append the std to the array
+        E = np.append(E, s)
+
+    # Calculate the standaard error
+    return E/np.sqrt(rows)
+
+def Dataframe_to_latex(dataframe, sep=","):
+    """
+    Function to convert a pandas datafrrame in a latex table
+
+    Input:
+    dataframe (pandas dataframe): The dataframe thats needs to be converted
+    sep (string): The seperator sign
+
+    """
+
+    # Create a string to store the latex table
+    latex_string = "" 
+
+    # Get the headers from pandas dataframe
+    headers = dataframe.columns
+
+    # Get the column count of the pandas dataframe
+    column_count = len(dataframe.columns)
+
+    # Add the top side of the pandas dataframe
+    latex_string = "\\begin{table}[h]\n  \\centering\n   \\begin{tabular}" + "{" + "c"*column_count + "}" + "\n"
+    
+    # Create a header variable
+    header = "       "
+
+    # Loop through all headers and add the column name to the header variabla
+    for i, h in enumerate(headers):
+
+        # Add column name to the header
+        header += h
+
+        # Check if it is the last column
+        if i != len(headers)-1:
+
+            # If not the last add &
+            header += " & "  
+        else:
+            # If last then add \\ to the end
+            header += "\\\ \n"
+
+    # Add the header to the dataframe
+    latex_string += header
+
+    # Add the hline to the latex string
+    latex_string += "       \\hline \n"
+
+
+    # Loop over all rows in the dataframe
+    for row in dataframe.itertuples(index=False):
+
+        # Create row string variable
+        row_string= "       "
+        
+        # Loop over all elements in a row
+        for i, element in enumerate(row):
+
+
+            # Add element to the row string
+            row_string += str(element).replace(".", sep)
+
+            # Check if it is the last element
+            if i != len(row)-1:
+
+                # If it is not the last element add &
+                row_string += " & "  
+            else:
+                # If it is the last element add \\
+                row_string += "\\\ \n"
+
+        # Add the row string to the latex string
+        latex_string += row_string
+    
+    # Add to botem of the latex table to the string
+    latex_string += "   \\end{tabular}\n   \\caption{Caption}\n   \\label{tab:my_label}\n\\end{table}"
+
+    # Print the string so the user can copy it
+    print(latex_string)
\ No newline at end of file
